@@ -630,7 +630,7 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 	struct ifaddrmsg *ifa;
 	struct priv *priv;
 #ifdef INET
-	struct in_addr addr, net, dest;
+	struct in_addr addr, net, brd;
 #endif
 #ifdef INET6
 	struct in6_addr addr6;
@@ -661,26 +661,29 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 	switch (ifa->ifa_family) {
 #ifdef INET
 	case AF_INET:
-		addr.s_addr = dest.s_addr = INADDR_ANY;
-		dest.s_addr = INADDR_ANY;
+		addr.s_addr = brd.s_addr = INADDR_ANY;
 		inet_cidrtoaddr(ifa->ifa_prefixlen, &net);
 		while (RTA_OK(rta, len)) {
 			switch (rta->rta_type) {
 			case IFA_ADDRESS:
 				if (ifp->flags & IFF_POINTOPOINT) {
-					memcpy(&dest.s_addr, RTA_DATA(rta),
-					       sizeof(addr.s_addr));
+					memcpy(&brd.s_addr, RTA_DATA(rta),
+					    sizeof(brd.s_addr));
 				}
 				break;
+			case IFA_BROADCAST:
+				memcpy(&brd.s_addr, RTA_DATA(rta),
+				    sizeof(brd.s_addr));
+				break;
 			case IFA_LOCAL:
 				memcpy(&addr.s_addr, RTA_DATA(rta),
-				       sizeof(addr.s_addr));
+				    sizeof(addr.s_addr));
 				break;
 			}
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv4_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr, &net, &dest, ifa->ifa_flags);
+		    &addr, &net, &brd, ifa->ifa_flags);
 		break;
 #endif
 #ifdef INET6
@@ -923,8 +926,6 @@ send_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp,
 	/* Request a reply */
 	hdr->nlmsg_flags |= NLM_F_ACK;
 	hdr->nlmsg_seq = (uint32_t)++ctx->seq;
-	if ((unsigned int)ctx->seq > UINT32_MAX)
-		ctx->seq = 0;
 	if (sendmsg(s, &msg, 0) != -1) {
 		ctx->sseq = ctx->seq;
 		r = get_netlink(ctx, ifp, s, 0, callback);
@@ -1283,10 +1284,15 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 	su.sll.sll_ifindex = (int)ifp->index;
 	su.sll.sll_hatype = htons(ifp->family);
 	su.sll.sll_halen = (unsigned char)ifp->hwlen;
-	if (ifp->family == ARPHRD_INFINIBAND)
+	if (ifp->family == ARPHRD_INFINIBAND) {
+		/* sockaddr_ll is not big enough for IPoIB which is why
+		 * sockaddr_storage is included in the union.
+		 * Ugly as sin, but it works. */
+		/* coverity[buffer_size] */
+		/* coverity[overrun-buffer-arg] */
 		memcpy(&su.sll.sll_addr,
 		    &ipv4_bcast_addr, sizeof(ipv4_bcast_addr));
-	else
+	} else
 		memset(&su.sll.sll_addr, 0xff, ifp->hwlen);
 	fd = ipv4_protocol_fd(ifp, protocol);
 
@@ -1611,12 +1617,10 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
 		    &rt->gate.s6_addr, sizeof(rt->gate.s6_addr));
 
-	if (!(rt->flags & RTF_REJECT)) {
-		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
-		if (rt->metric)
-			add_attr_32(&nlm.hdr, sizeof(nlm),
-			    RTA_PRIORITY, rt->metric);
-	}
+	add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
+	if (rt->metric)
+		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_PRIORITY, rt->metric);
+
 	if (cmd != RTM_DELETE && rt->mtu) {
 		char metricsbuf[32];
 		struct rtattr *metrics = (void *)metricsbuf;
